/* Our dimensions are
 * D1 - Fan (hit)
 * D2 - Channel (user)
 * D3 - Quiz Type (hit)
 * D4 - Quiz complexity (hit)
 */

/*  Our Metrics are
 *  M1 - Score
 *  M2 - Fan Action (count of actions performed by Fan so that 
 *  we can measure their stickiness
 */

/*  Category, action, label, value */
/* capturing analytics on fan behavior  */
(function (namespace) {
    var myfantainid = null;
    var timerbucket = {};

    namespace.GAsuccessHandler = function (result) {
        fg.debugMsg('GA Successful: ' + result, fg.debugMsg.msgs);
    }

    namespace.GAerrorHandler = function (error) {
        fg.debugMsg('GA Error: ' + error, fg.debugMsg.errors);
    }

    namespace.sendevent = function (params) {
            if (fg.isApp)
                fg.gaPlugin.trackEvent(fg.GAsuccessHandler, 
                    namespace.GAerrorHandler,
                    params[0], params[1], 
                    params[2], params[3]);
            else
                typeof ga != 'undefined' && ga('send', 'event',
                    params[0], params[1], 
                    params[2], params[3]);
    }

    namespace.sendpageview = function(pagename) {
            if (fg.isApp)
                fg.gaPlugin.trackPage(namespace.GAsuccessHandler,
                        namespace.GAerrorHandler, pagename);
            else
                typeof ga != 'undefined' && ga('send', 'pageview', pagename);

    }

    namespace.setvar = function(key, value) {
        if (fg.isApp)
            fg.gaPlugin.setVariable(
                    namespace.GAsuccessHandler,
                    namespace.GAerrorHandler,
                    key, value);
        else
            typeof ga != 'undefined' && ga('send', key, value);
    }

    namespace.sendtiming = function(category, name,
                         timing) {
            if (fg.isApp)
                fg.gaPlugin.trackTiming(
                        namespace.GAsuccessHandler,
                        namespace.GAerrorHandler,
                        category, name,
                        timing);
            else
                typeof ga != 'undefined' && ga('send',
                   'timing',
                   category,
                   name,
                   timing);
    }

    namespace.sendsocial = function(network, action, url) {
            if (fg.isApp)
                fg.gaPlugin.trackSocial(namespace.GAsuccessHandler,
                        namespace.GAerrorHandler,
                        network,
                        action,
                        url);
            else
                typeof ga != 'undefined' && ga('send', 'social',
                        network,
                        action,
                        url);
        }

    namespace.init = function() {
        if (fg.isApp) {
            fg.gaPlugin.init( namespace.GAsuccessHandler,
                              namespace.GAerrorHandler,
                              fg.gaID, 10);

            namespace.setvar('dimension2', 'App');
            namespace.setvar('appName', fg.installation + ' App');
        }
        else {
            typeof ga != 'undefined' && ga('create', fg.gaID, 'auto');
            namespace.setvar('dimension2', 'Web');
        }
    }

    namespace.login = function(fantainid) {
        myfantainid = fantainid;
    }

    namespace.page = function(pagename) {
        if (myfantainid != null)
            namespace.setvar('dimension1', myfantainid);
        namespace.sendpageview(pagename);
    }


    namespace.fan = function(action, param) {
        /*  action  is one of
         *  score : score update (param = score)
         *  quiz :  a Quiz was responded to by the fan (param = quizid)
         */
        if (myfantainid != null)
            namespace.setvar('dimension1', myfantainid);
        namespace.sendevent(['fan', action, param]);
    }

    namespace.ui = function(action, param, label) {
        if (myfantainid != null)
            namespace.setvar('dimension1', myfantainid);
        namespace.sendevent(['ui', action, param, label]);

    }

    /* Call this to send a Quiz related event.  This is not used to connect a
     * quiz and a fan. This is just to track quiz metrics on total new / closed
     * etc.,
     * action is one of
     * new - a new quiz (count is not relevant)
     * closed - quiz was closed (count = number of respondents)
     */
    namespace.quiz = function(quizid, qaction, qtype, complexity)  {
        /*
         * action should be one of received, responded, updated, closed
         */
        if (typeof qtype != 'undefined')
            namespace.setvar('dimension3', qtype);
        if (typeof complexity != 'undefined')
            namespace.setvar('dimension4', complexity);
        namespace.sendevent(['quiz', qaction, 'QUIZ_' + quizid]);
    }


    /*  Function that will help you generate analytics on errors.  
     *  Use this to report business errors only not tech errors
    namespace.error = function(msg) {
        ga('send', 'exception', {
            'exDescription' : msg
        });
    }
   
   /* Function to record crashes.  Use this to report both business and
    * technology application crashes
    namespace.crash = function(msg) {
        myga('send', 'exception', {
            'exDescription' : msg,
            'exFatal' : true
        });
    }
    */

    getRandomInt = function(min, max) {
        return Math.floor(Math.random() * (max - min + 1)) + min;
    }

    namespace.timerstart = function(cat, name) {
        timenow = Date.now();
        timerbucket[cat + name + timenow] = { 'cat' : cat, 'name' : name, 'start' : timenow };
        return cat + name + timenow;
    }

    namespace.timerstop = function(unqid) {
        if (myfantainid != null)
            namespace.setvar('dimension1', myfantainid);

        timenow = Date.now();
        if (typeof timerbucket[unqid] != 'undefined') {
            delta = timenow - timerbucket[unqid]
            namespace.sendtiming(timerbucket[unqid].cat,
                       timerbucket[unqid].name,
                       delta);
            delete timerbucket[unqid];
        }
    }

    namespace.timing = function(cat, name, timing) {
        if (myfantainid != null)
            namespace.setvar('dimension1', myfantainid);
        namespace.sendtiming(cat, name, timing);
    }

    namespace.facebook = function(action, url) {
        if (myfantainid != null)
            namespace.setvar('dimension1', myfantainid);
        namespace.sendsocial('Facebook', action, url);
    }
})(fantain.analytics);
