/**
 * Created with JetBrains WebStorm.
 * User: aravind
 * Date: 10/7/13
 * Time: 3:05 PM
 * To change this template use File | Settings | File Templates.
 */

/* ----------------------------------------------------------------------------------
 Device access functions
 ---------------------------------------------------------------------------------- */

define(function () {

    return function deviceAccess() {

        fg.debugMsg('Device access file loaded...');

        var self = this;

        self.connectionStates = {
            'UNKNOWN': 'Unknown',
            'ETHERNET': 'Ethernet',
            'WIFI': 'WiFi',
            'CELL_2G': 'Cell 2g',
            'CELL_3G': 'Cell 3g',
            'CELL_4G': 'Cell 4g',
            'CELL': 'Cell generic',
            'NONE': 'No network',
            '3G': 'Emulator 3g'
        };

        self.fantainUp = function () {
            if (fg.isApp && self.connection() == self.connectionStates.NONE) {
                fg.online = false; fg.alive = false;
                fg.is(fg.fVM.online) ? fg.fVM.online(fg.online) : '';
                fg.is(fg.fVM.alive)  ? fg.fVM.alive(fg.alive)  : '';
                fg.connectionCheckRan = true;
            } else {
                fg.online = true;
                fg.is(fg.fVM.online) ? fg.fVM.online(fg.online) : '';
                fg.debugMsg('About to ping for server status ...', fg.debugLevel.msgs);
                requirejs(['fantainUtils-svc'], function(svc) {
                    var s = new svc();
                    s.svcCall({url: fg.svcPath + 'ping',type:'get',returnNode:'pong', fantainSvc: true,
                        successCallback: self.pingSuccess, errorCallback: self.pingError, wait:true});
                });
            }
        };

        self.pingSuccess = function( pong ) {
            fg.debugMsg('Fantain Servers ready: ' + pong.msg, fg.debugLevel.msgs);
            fg.alive = true;
            fg.is(fg.fVM.alive)  ? fg.fVM.alive(fg.alive)  : '';
            //This is for situations when we come back from the dead
            fg.is(fg.fVM.gameOnCalled) && fg.fVM.reconnectRT(); 
            fg.pingError = '';
            fg.connectionCheckRan = true;
            fg.serverVersionCode = pong.versioncode;
        };

        self.pingError = function ( e ) {
            fg.debugMsg('Fantain Servers unreachable' + (e ? ': Timeout' : ''), fg.debugLevel.msgs);
            fg.serverVersionCode = 1; //So any current version will work
            fg.alive = false;
            fg.is(fg.fVM.alive)  ? fg.fVM.alive(fg.alive)  : '';
            fg.pingError = e.timeout ? 'Timeout' : e.msg;
            fg.connectionCheckRan = true;
        };


        self.connection = function () {
            var networkState = navigator.connection.type;
            fg.debugMsg("networkState is " + networkState, fg.debugLevel.msgs);
            fg.debugMsg('Connection type: ' + self.connectionStates[networkState.trim().toUpperCase()], fg.debugLevel.msgs);
            return self.connectionStates[networkState.trim().toUpperCase()];
        };

        self.dev = function () {
            var devDetails = {};
            devDetails["name"] = device.name;
            devDetails["cordova"] = device.cordova;
            devDetails["platform"] = device.platform;
            devDetails["uuid"] = device.uuid;
            devDetails["model"] = device.model;
            devDetails["version"] = device.version;
            return devDetails;
        };

        self.geo = function () {

        	fg.debugMsg('About to get geolocation',fg.debugLevel.msgs);
            navigator.geolocation.getCurrentPosition(self.geoSuccess, self.geoError);

        };

        self.geoSuccess = function (position) {
        	fg.debugMsg('geolocation success',fg.debugLevel.msgs);
            var x = {};
            x["status"] = "success";
            x["latitude"] = position.coords.latitude;
            x["longitude"] = position.coords.longitude;
            x["altitude"] = position.coords.altitude;
            x["accuracy"] = position.coords.accuracy;
            x["altitudeAccuracy"] = position.coords.altitudeAccuracy;
            x["heading"] = position.coords.heading;
            x["speed"] = position.coords.speed;
            x["timeStamp"] = position.timeStamp;
            self.getloc(x);
        };
        self.geoError = function (error) {
        	fg.debugMsg('geolocation error',fg.debugLevel.errors);
            var x = {};
            x["status"] = "error";
            x["errorCode"] = error.errorCode;
            x["message"] = error.message;
            self.getloc(x);
        };
        
        /* Throw out some device properties to make sure it works */
        self.setDevDetails = function () {
            fg.debugMsg('Connection: ' + self.connection(), fg.debugLevel.msgs);
            switch (self.connection()) {
                case "Ethernet":
                case "WiFi":
                case "Cell 3g":
                case "Cell 4g":
                case "Emulator 3g":
                    fg.svcTimeout = 10 * 1000;
                    break;
                case "Cell 2g":
                    fg.svcTimeout = 20 * 1000;
                    break;
                case "Cell generic":
                case "No Network":
                case "Unknown":
                    fg.svcTimeout = 30 * 1000;
                    break;
                default:
                    fg.svcTimeout = 30 * 1000;
            }
            fg.debugMsg('Device name: ' + self.dev().name, fg.debugLevel.msgs);
            fg.debugMsg('Device platform: ' + self.dev().platform, fg.debugLevel.msgs);
            fg.debugMsg('Device model: ' + self.dev().model, fg.debugLevel.msgs);
            self.geo();
        };

        self.getloc = function (x) {
            if (x.status == "success") {
                fg.debugMsg('Latitude: ' + x.latitude + ' & Longitude: ' + x.longitude, fg.debugLevel.msgs);
                //alert('Latitude: ' + x.latitude + ' & Longitude: ' + x.longitude);
                fg.geoloc.lat = x.latitude;
                fg.geoloc.lng = x.longitude;
            } else {
                fg.debugMsg('Geo Error: ' + x.message, fg.debugLevel.msgs);
                //alert('Geo Error: ' + x.message);
            }
        };
    };
});


